package be.iminds.ilabt.jfed.espec.util;

import javax.annotation.Nonnull;

/**
 * LimitedLiveLog: A log that gets updated. It will automatically limited in size.
 * You can subscribe to listen for changes.
 */
public interface LimitedLiveLog {
    interface ChangeListener {
        void onChanged(@Nonnull LimitedLiveLog source, @Nonnull String addedText);
        void onEndReached(@Nonnull LimitedLiveLog source);
    }

    void addChangeListener(@Nonnull ChangeListener l);
    void removeChangeListener(@Nonnull ChangeListener l);

    void appendText(@Nonnull String extraText);
    void setEndReached();

    /** returns the full text, possibly with ... somewhere in the middle */
    @Nonnull String getText();

    /**
     * @return the head part of the log (without ...). This returns the same as getText() if the limit is not reached.
     */
    @Nonnull String getHeadText();
    /**
     * @return the tail part of the log (without ...) or an empty String if the limit is not reached.
     */
    @Nonnull String getTailText();

    /**
     * @return true if the limit is reached and text has been discarded, false otherwise.
     */
    boolean isLimited();

    /**
     *
     * @return The total current size of all text received. This includes any text that has been discarded because of sotrage limits.
     */
    int getReceivedSize();

    /**
     *
     * @return The total current size of the stored text (without any ..., thus this equals head.size + tail.size)
     */
    int getStoredSize();

    /**
     * @return The maximum ammount of text that will be stored.
     */
    int getSizeLimit();

    boolean isEndReached();
}
